from outline_cli.outline import OutlineVPN

from .helper import (
    execute_method,
    get_config_from_app_ini,
    get_method_arguments,
    get_method_user_want_to_call,
    get_public_methods,
)


def init_outline():
    return OutlineVPN(
        certSha256=get_config_from_app_ini("OutlineVPN", "certSha256"),
        apiUrl=get_config_from_app_ini("OutlineVPN", "apiUrl"),
    )


def init_cli():
    outline_client = init_outline()
    return outline_client, get_public_methods(outline_client)


def start_cli():
    outline_client, outline_methods = init_cli()
    method_user_want_to_call = get_method_user_want_to_call(outline_methods)

    method_arguments = get_method_arguments(outline_client, method_user_want_to_call)
    execute_method(outline_client, method_user_want_to_call, method_arguments)



import configparser
import pkgutil
from inspect import getfullargspec

from PyInquirer import prompt

from outline_cli.gmail import Gmail

config = configparser.ConfigParser()
config.read("app.ini", encoding="utf-8")


def get_config_from_app_ini(section_name, key):
    return config.get(section_name, key)


def get_public_methods(instance):
    return [
        method
        for method in dir(instance)
        if callable(getattr(instance, method)) and not method.startswith("_")
    ]


def get_method_user_want_to_call(methods):
    return prompt(
        [
            {
                "type": "list",
                "name": "method",
                "message": "Please choose which method you want to call? ",
                "choices": methods,
            }
        ]
    )["method"]


def get_method_arguments(instance, method):
    method_arguments = getfullargspec(getattr(instance, method)).args
    method_arguments.pop(0)  # NOTE: Pop 'self'
    return method_arguments


def execute_method(outline_client, method_user_want_to_call, method_arguments):
    if len(method_arguments) > 0:
        print(
            getattr(outline_client, method_user_want_to_call)(
                **prompt(
                    [
                        {"type": "input", "name": arg, "message": f"{arg} :"}
                        for arg in method_arguments
                    ]
                )
            )
        )
    else:
        print(getattr(outline_client, method_user_want_to_call)())


def get_email_list_from_file(filename):
    with open(filename, "r") as file:
        return [email.strip() for email in file.readlines()]


def get_username_by_email(email):
    # NOTE: first_name.last_name@gmail.com
    return " ".join([c.capitalize() for c in email.split("@")[0].split(".")])


def send_email(email, username, accessUrl):
    gmail_user = get_config_from_app_ini("Gmail", "EMAIL")
    gmail_password = get_config_from_app_ini("Gmail", "APP_PASSWORD")

    gmail = Gmail(gmail_user, gmail_password)
    gmail.draft(
        to_email=email,
        message=pkgutil.get_data(__package__, "templates/mail_template.txt")
        .decode()
        .format(username=username, accessUrl=accessUrl),
    )
    gmail.send()
    gmail.quit()

    print(f"Email sent to {email}")



import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText


class Gmail:
    def __init__(self, user, password):
        self.user = user
        self.password = password
        self.init_server()
        self.init_message()

    def init_server(self):
        self.server = smtplib.SMTP_SSL("smtp.gmail.com", 465)
        self.__ehlo()
        self.__login()

    def init_message(self):
        self.message = MIMEMultipart()

    def __ehlo(self):
        self.server.ehlo()

    def __login(self):
        self.server.login(self.user, self.password)

    def send(self):
        self.server.send_message(self.message)

    def quit(self):
        self.server.quit()

    def draft(self, to_email, message, subject="VPN 開通"):
        self.message["Subject"] = subject
        self.message["From"] = self.user
        self.message["To"] = to_email
        self.message.preamble = "Multipart massage.\n"
        part = MIMEText(message)
        self.message.attach(part)



import unittest
from unittest.mock import MagicMock, patch

from requests.exceptions import MissingSchema

import outline_cli
from outline_cli.gmail import Gmail
from outline_cli.helper import (
    get_config_from_app_ini,
    get_email_list_from_file,
    get_username_by_email,
    send_email,
)
from outline_cli.outline import OutlineVPN


class TestOutlineVPN(unittest.TestCase):
    """Test Outline VPN"""

    @classmethod
    def setUpClass(cls):
        cls.client = OutlineVPN("test_certSha256", "https://test_apiUrl")

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_get_server_info(self, mock):
        mock.return_value = "ok"
        self.client.get_server_info()
        mock.assert_called_once_with("/server", "GET", {})

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_create_access_key(self, mock):
        mock.return_value = "ok"
        self.client.create_access_key()
        mock.assert_called_once_with("/access-keys", "POST", {})

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_list_access_keys(self, mock):
        mock.return_value = "ok"
        self.client.list_access_keys()
        mock.assert_called_once_with("/access-keys", "GET", {})

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_delete_access_key(self, mock):
        mock.return_value = "ok"
        self.client.delete_access_key(id=1)
        mock.assert_called_once_with("/access-keys/1", "DELETE", {})

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_rename_access_key(self, mock):
        mock.return_value = "ok"
        self.client.rename_access_key(id=1, username="Jimmy Chen")
        mock.assert_called_once_with(
            "/access-keys/1/name", "PUT", {"name": "Jimmy Chen"}
        )

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_get_each_access_key_data_transferred(self, mock):
        mock.return_value = "ok"
        self.client.get_each_access_key_data_transferred()
        mock.assert_called_once_with("/metrics/transfer", "GET", {})

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    @patch("outline_cli.outline.send_email")
    def test_create_user_by_email(self, send_email_mock, outline_mock):
        outline_mock.return_value = '{"id": 1, "accessUrl": 1}'
        send_email_mock.return_value = "Success"
        self.client.create_user_by_email("test.chen@gmail.com")
        self.assertEqual(outline_mock.call_count, 2)
        self.assertEqual(send_email_mock.call_count, 1)

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_change_hostname_for_access_keys(self, mock):
        mock.return_value = "ok"
        self.client.change_hostname_for_access_keys("127.0.0.1")
        mock.assert_called_once_with(
            "/server/hostname-for-access-keys", "PUT", {"hostname": "127.0.0.1"}
        )

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_rename_server(self, mock):
        mock.return_value = "ok"
        self.client.rename_server("test server")
        mock.assert_called_once_with("/name", "PUT", {"name": "test server"})

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_get_whether_metrics_is_being_shared(self, mock):
        mock.return_value = "ok"
        self.client.get_whether_metrics_is_being_shared()
        mock.assert_called_once_with("/metrics/enabled", "GET", {})

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_enable_or_disable_sharing_metrics(self, mock):
        mock.return_value = "ok"
        self.client.enable_or_disable_sharing_metrics(True)
        mock.assert_called_once_with(
            "/metrics/enabled", "PUT", {"metricsEnabled": True}
        )

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_change_default_port_for_newly_created_access(self, mock):
        mock.return_value = "ok"
        self.client.change_default_port_for_newly_created_access("7004")
        mock.assert_called_once_with(
            "/server/port-for-new-access-keys", "PUT", {"port": "7004"}
        )

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_set_data_transfer_limit_for_all_access_keys(self, mock):
        mock.return_value = "ok"
        self.client.set_data_transfer_limit_for_all_access_keys(1234)
        mock.assert_called_once_with(
            "/experimental/access-key-data-limit", "PUT", {"limit": {"bytes": 1234}}
        )

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    def test_remove_access_key_data_limit(self, mock):
        mock.return_value = "ok"
        self.client.remove_access_key_data_limit()
        mock.assert_called_once_with(
            "/experimental/access-key-data-limit", "DELETE", {}
        )

    @patch.object(OutlineVPN, "_OutlineVPN__call_api")
    @patch("outline_cli.outline.send_email")
    def test_batch_create_user_by_email_list(self, send_email_mock, outline_mock):
        outline_mock.return_value = '{"id": 1, "accessUrl": 1}'
        send_email_mock.return_value = "Success"
        self.client.batch_create_user_by_email_list("example_email_list.txt")
        self.assertEqual(outline_mock.call_count, 20)
        self.assertEqual(send_email_mock.call_count, 10)


class TestHelper(unittest.TestCase):
    """Test helper"""

    def test_get_config_from_app_ini(self):
        self.assertEqual(get_config_from_app_ini("Gmail", "EMAIL"), "your_email")

    def test_get_email_list_from_file(self):
        self.assertEqual(
            get_email_list_from_file("example_email_list.txt"),
            [
                "user1@gmail.com",
                "user2@gmail.com",
                "user3@gmail.com",
                "user4@gmail.com",
                "user5@gmail.com",
                "user6@gmail.com",
                "user7@gmail.com",
                "user8@gmail.com",
                "user9@gmail.com",
                "user10@gmail.com",
            ],
        )

    def test_get_username_by_email(self):
        self.assertEqual(get_username_by_email("jimmy.chen@gmail.com"), "Jimmy Chen")

    @patch("outline_cli.helper.Gmail")
    def test_send_email(self, mock):
        mock.init_server = MagicMock()
        mock.send = MagicMock()
        send_email(
            "jimmy.chen@gmail.com", "Jimmy Chen", {"accessUrl": "test accessUrl"}
        )


class TestGmail(unittest.TestCase):
    """Test gmail"""

    @patch.object(Gmail, "_Gmail__login")
    def test_gmail(self, mock):
        server = Gmail("user", "password")
        self.assertEqual(mock.call_count, 1)
        server.draft("test@gmail.com", "Hello world", "Subject")
        server.send = MagicMock()
        server.send()
        server.quit()


class TestMain(unittest.TestCase):
    """Test main"""

    def test_init_outline(self):
        self.assertIsInstance(outline_cli.init_outline(), OutlineVPN)

    def test_init_cli(self):
        client, public_methods = outline_cli.init_cli()
        self.assertIsInstance(client, OutlineVPN)
        self.assertEqual(
            public_methods,
            [
                "batch_create_user_by_email_list",
                "change_default_port_for_newly_created_access",
                "change_hostname_for_access_keys",
                "create_access_key",
                "create_user_by_email",
                "delete_access_key",
                "enable_or_disable_sharing_metrics",
                "get_each_access_key_data_transferred",
                "get_server_info",
                "get_whether_metrics_is_being_shared",
                "list_access_keys",
                "remove_access_key_data_limit",
                "rename_access_key",
                "rename_server",
                "set_data_transfer_limit_for_all_access_keys",
            ],
        )

    @classmethod
    @patch("outline_cli.helper.prompt")
    def test_start_cli_with_no_args_method(cls, mock):
        mock.return_value = {"method": "get_server_info"}
        try:
            outline_cli.start_cli()
        except MissingSchema:
            pass

    @classmethod
    @patch("outline_cli.helper.prompt")
    def test_start_cli_with_args_method(cls, mock):
        mock.return_value = {"method": "batch_create_user_by_email_list"}
        try:
            outline_cli.start_cli()
        except (MissingSchema, TypeError):
            pass



import json

import requests
from requests_toolbelt.adapters.fingerprint import FingerprintAdapter

from outline_cli.helper import (
    get_email_list_from_file,
    get_username_by_email,
    send_email,
)


class OutlineVPN:
    def __init__(self, certSha256="", apiUrl=""):
        self.certSha256 = certSha256
        self.apiUrl = apiUrl
        self.__init_requests_session()

    def __init_requests_session(self):
        self.session = requests.Session()
        self.session.mount(self.apiUrl, FingerprintAdapter(self.certSha256))

    def __call_api(self, api_path, request_method, request_body):
        response = self.session.request(
            method=request_method,
            url=(self.apiUrl + api_path),
            data=request_body,
            verify=False,
        )
        return response.text

    def get_server_info(self):
        return self.__call_api("/server", "GET", {})

    def change_hostname_for_access_keys(self, hostname):
        return self.__call_api(
            "/server/hostname-for-access-keys", "PUT", {"hostname": hostname}
        )

    def rename_server(self, name):
        return self.__call_api("/name", "PUT", {"name": name})

    def get_whether_metrics_is_being_shared(self):
        return self.__call_api("/metrics/enabled", "GET", {})

    def enable_or_disable_sharing_metrics(self, metricsEnabled):
        return self.__call_api(
            "/metrics/enabled", "PUT", {"metricsEnabled": metricsEnabled}
        )

    def change_default_port_for_newly_created_access(self, port):
        return self.__call_api(
            "/server/port-for-new-access-keys", "PUT", {"port": port}
        )

    def create_access_key(self):
        return self.__call_api("/access-keys", "POST", {})

    def list_access_keys(self):
        return self.__call_api("/access-keys", "GET", {})

    def delete_access_key(self, id):
        return self.__call_api(f"/access-keys/{id}", "DELETE", {})

    def rename_access_key(self, id, username):
        return self.__call_api(f"/access-keys/{id}/name", "PUT", {"name": username})

    def get_each_access_key_data_transferred(self):
        return self.__call_api("/metrics/transfer", "GET", {})

    def set_data_transfer_limit_for_all_access_keys(self, byte):
        return self.__call_api(
            "/experimental/access-key-data-limit", "PUT", {"limit": {"bytes": byte}}
        )

    def remove_access_key_data_limit(self):
        return self.__call_api("/experimental/access-key-data-limit", "DELETE", {})

    def create_user_by_email(self, email):
        username = get_username_by_email(email)
        resp = json.loads(self.create_access_key())
        self.rename_access_key(resp["id"], username)
        send_email(email, username, resp["accessUrl"])
        return resp

    def batch_create_user_by_email_list(self, email_list_filename):
        email_list = get_email_list_from_file(email_list_filename)
        for email in email_list:
            self.create_user_by_email(email)
        return "Finish"



import os

from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = "\n" + f.read()

setup(
    name="outline-cli",
    use_scm_version=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Jimmy Chen",
    author_email="jimmychen260@gmail.com",
    project_urls={
        "Bug Tracker": "https://github.com/JMCFTW/outline-cli/issues",
        "Source Code": "https://github.com/JMCFTW/outline-cli/",
    },
    setup_requires=["setuptools_scm"],
    packages=["outline_cli"],
    package_dir={"": "src"},
    entry_points={"console_scripts": ["outline-cli=outline_cli:start_cli"]},
    install_requires=[
        "certifi==2020.6.20",
        "chardet==3.0.4",
        "idna==2.10",
        "requests==2.24.0",
        "requests-toolbelt==0.9.1",
        "urllib3==1.25.11",
        "PyInquirer==1.0.3",
    ],
    zip_safe=True,
    include_package_data=True,
    python_requires=">3.6",
)
